package com.nokia.hb.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.WebSocketMessage;
import org.springframework.web.socket.WebSocketSession;

import java.util.Map;

/**
 * 网络套接字服务
 */
@Slf4j
@Service
public class WebSocketService implements WebSocketHandler {
    private final SessionService sessionService;

    public WebSocketService(SessionService sessionService) {
        this.sessionService = sessionService;
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        String sessionId = (String) attributes.get("JSESSIONID");
        String username = (String) attributes.get("username");
        log.info("websocket connection established: {} -> {} -> {}", session.getId(), sessionId, username);
    }

    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        String sessionId = (String) attributes.get("JSESSIONID");
        String username = (String) attributes.get("username");
        log.info("receive websocket message: {} -> {} -> {} -> {}", session.getId(), sessionId, username,
                message.getPayload());
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        String sessionId = (String) attributes.get("JSESSIONID");
        String username = (String) attributes.get("username");
        log.error("websocket transport error: {} -> {} -> {} -> {}", session.getId(), sessionId, username,
                exception.getMessage(), exception);
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        Map<String, Object> attributes = session.getAttributes();
        String sessionId = (String) attributes.get("JSESSIONID");
        String username = (String) attributes.get("username");
        log.info("websocket connection closed: {}, {} -> {} -> {}", closeStatus, session.getId(), sessionId, username);
        sessionService.invalidate(username);
    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }
}
